@@ -56,75 +56,6 @@ tracker_is_blank_string (const char *str)
 	return TRUE;
 }
 
-/* Removes a substring modifing haystack in place */
-gchar *
-tracker_string_remove (gchar       *haystack,
-                       const gchar *needle)
-{
-	gchar *current, *pos, *next, *end;
-	gint len;
-
-	len = strlen (needle);
-	end = haystack + strlen (haystack);
-	current = pos = strstr (haystack, needle);
-
-	if (!current) {
-		return haystack;
-	}
-
-	while (*current != '\0') {
-		pos = strstr (pos, needle) + len;
-		next = strstr (pos, needle);
-
-		if (!next) {
-			next = end;
-		}
-
-		while (pos < next) {
-			*current = *pos;
-			current++;
-			pos++;
-		}
-
-		if (*pos == '\0') {
-			*current = *pos;
-		}
-	}
-
-	return haystack;
-}
-
-gchar *
-tracker_string_replace (const gchar *haystack,
-                        const gchar *needle,
-                        const gchar *replacement)
-{
-	GString *str;
-	gint     pos, needle_len;
-
-	g_return_val_if_fail (haystack != NULL, NULL);
-	g_return_val_if_fail (needle != NULL, NULL);
-
-	needle_len = strlen (needle);
-
-	str = g_string_new ("");
-
-	/* FIXME: should use strstr */
-	for (pos = 0; haystack[pos]; pos++) {
-		if (strncmp (&haystack[pos], needle, needle_len) == 0) {
-			if (replacement) {
-				str = g_string_append (str, replacement);
-			}
-
-			pos += needle_len - 1;
-		} else {
-			str = g_string_append_c (str, haystack[pos]);
-		}
-	}
-
-	return g_string_free (str, FALSE);
-}
-
 gchar *
 tracker_seconds_estimate_to_string (gdouble  seconds_elapsed,
                                     gboolean short_string,
