@@ -126,7 +126,8 @@ void
 tracker_data_backup_restore (GFile *journal,
                              TrackerDataBackupFinished callback,
                              gpointer user_data,
-                             GDestroyNotify destroy)
+                             GDestroyNotify destroy,
+                             const gchar **test_schemas)
 {
 	BackupSaveInfo *info;
 
@@ -153,7 +154,7 @@ tracker_data_backup_restore (GFile *journal,
 		             &info->error);
 
 		tracker_db_manager_init_locations ();
-		tracker_db_journal_init (NULL);
+		tracker_db_journal_init (NULL, FALSE);
 
 		if (info->error) {
 			tracker_db_manager_restore_from_temp ();
@@ -163,13 +164,13 @@ tracker_data_backup_restore (GFile *journal,
 
 		tracker_db_journal_shutdown ();
 
-		tracker_data_manager_init (flags, NULL, &is_first);
+		tracker_data_manager_init (flags, test_schemas, &is_first, TRUE);
 
 	} else {
 		g_set_error (&info->error, TRACKER_DATA_BACKUP_ERROR, 0, 
 		             "Backup file doesn't exist");
 	}
 
-	g_idle_add (on_restore_done, info);
+	on_restore_done (info);
 }
 
