@@ -1,4 +1,3 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2006, Mr Jamie McCracken (jamiemcc@gnome.org)
  * Copyright (C) 2008, Nokia (urho.konttori@nokia.com)
@@ -24,7 +23,7 @@
 
 #include <glib.h>
 
-#include <libtracker-common/tracker-ontology.h>
+#include <libtracker-common/tracker-ontologies.h>
 
 #include <libtracker-db/tracker-db-interface.h>
 
@@ -40,7 +39,8 @@ typedef enum  {
 	TRACKER_DATA_ERROR_UNKNOWN_CLASS,
 	TRACKER_DATA_ERROR_UNKNOWN_PROPERTY,
 	TRACKER_DATA_ERROR_INVALID_TYPE,
-	TRACKER_DATA_ERROR_CONSTRAINT
+	TRACKER_DATA_ERROR_CONSTRAINT,
+	TRACKER_DATA_ERROR_NO_SPACE
 } TrackerDataError;
 
 typedef void (*TrackerStatementCallback) (const gchar *graph,
@@ -55,7 +55,7 @@ GQuark   tracker_data_error_quark                   (void);
 
 /* Metadata */
 void     tracker_data_delete_resource_description   (const gchar               *graph,
-                                                     const gchar               *uri,
+                                                     const gchar               *url,
                                                      GError                   **error);
 void     tracker_data_delete_statement              (const gchar               *graph,
                                                      const gchar               *subject,
@@ -77,21 +77,24 @@ void     tracker_data_insert_statement_with_string  (const gchar               *
                                                      const gchar               *predicate,
                                                      const gchar               *object,
                                                      GError                   **error);
-void     tracker_data_begin_transaction             (void);
-void     tracker_data_commit_transaction            (void);
+void     tracker_data_begin_db_transaction          (void);
+void     tracker_data_begin_db_transaction_for_replay (time_t                   time);
+void     tracker_data_commit_db_transaction         (void);
+void     tracker_data_begin_transaction             (GError                   **error);
+void     tracker_data_commit_transaction            (GError                   **error);
+void     tracker_data_rollback_transaction          (void);
 void     tracker_data_update_sparql                 (const gchar               *update,
                                                      GError                   **error);
 GPtrArray *
-tracker_data_update_sparql_blank           (const gchar               *update,
-                                            GError                   **error);
+         tracker_data_update_sparql_blank           (const gchar               *update,
+                                                     GError                   **error);
 void     tracker_data_update_buffer_flush           (GError                   **error);
+void     tracker_data_update_buffer_might_flush     (GError                   **error);
 
-/* Volume handling */
-void     tracker_data_update_enable_volume          (const gchar               *udi,
-                                                     const gchar               *mount_path);
-void     tracker_data_update_disable_volume         (const gchar               *udi);
-void     tracker_data_update_disable_all_volumes    (void);
-void     tracker_data_update_reset_volume           (const gchar               *uri);
+void     tracker_data_sync                          (void);
+void     tracker_data_replay_journal                (GHashTable                *classes,
+                                                     GHashTable                *properties,
+                                                     GHashTable                *id_uri_map);
 
 /* Calling back */
 void     tracker_data_add_insert_statement_callback   (TrackerStatementCallback   callback,
@@ -103,6 +106,9 @@ void     tracker_data_add_commit_statement_callback   (TrackerCommitCallback
 void     tracker_data_add_rollback_statement_callback (TrackerCommitCallback      callback,
                                                        gpointer                   user_data);
 
+void     tracker_data_update_shutdown                 (void);
+#define  tracker_data_update_init                     tracker_data_update_shutdown
+
 G_END_DECLS
 
 #endif /* __LIBTRACKER_DATA_UPDATE_H__ */
