@@ -1,4 +1,3 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * Copyright (C) 2008 Nokia
  *
@@ -29,10 +28,10 @@ G_BEGIN_DECLS
 #error "only <libtracker-db/tracker-db.h> must be included directly."
 #endif
 
-#define TRACKER_TYPE_DB_INTERFACE_SQLITE	 (tracker_db_interface_sqlite_get_type ())
-#define TRACKER_DB_INTERFACE_SQLITE(o)		 (G_TYPE_CHECK_INSTANCE_CAST ((o), TRACKER_TYPE_DB_INTERFACE_SQLITE, TrackerDBInterfaceSqlite))
-#define TRACKER_DB_INTERFACE_SQLITE_CLASS(c)	 (G_TYPE_CHECK_CLASS_CAST ((c),    TRACKER_TYPE_DB_INTERFACE_SQLITE, TrackerDBInterfaceSqliteClass))
-#define TRACKER_IS_DB_INTERFACE_SQLITE(o)	 (G_TYPE_CHECK_INSTANCE_TYPE ((o), TRACKER_TYPE_DB_INTERFACE_SQLITE))
+#define TRACKER_TYPE_DB_INTERFACE_SQLITE         (tracker_db_interface_sqlite_get_type ())
+#define TRACKER_DB_INTERFACE_SQLITE(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), TRACKER_TYPE_DB_INTERFACE_SQLITE, TrackerDBInterfaceSqlite))
+#define TRACKER_DB_INTERFACE_SQLITE_CLASS(c)     (G_TYPE_CHECK_CLASS_CAST ((c),    TRACKER_TYPE_DB_INTERFACE_SQLITE, TrackerDBInterfaceSqliteClass))
+#define TRACKER_IS_DB_INTERFACE_SQLITE(o)        (G_TYPE_CHECK_INSTANCE_TYPE ((o), TRACKER_TYPE_DB_INTERFACE_SQLITE))
 #define TRACKER_IS_DB_INTERFACE_SQLITE_CLASS(c)  (G_TYPE_CHECK_CLASS_TYPE ((o),    TRACKER_TYPE_DB_INTERFACE_SQLITE))
 #define TRACKER_DB_INTERFACE_SQLITE_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o),  TRACKER_TYPE_DB_INTERFACE_SQLITE, TrackerDBInterfaceSqliteClass))
 
@@ -40,20 +39,12 @@ typedef struct TrackerDBInterfaceSqlite      TrackerDBInterfaceSqlite;
 typedef struct TrackerDBInterfaceSqliteClass TrackerDBInterfaceSqliteClass;
 
 typedef gint (* TrackerDBCollationFunc) (gchar *str1,
-					 gint	len1,
-					 gchar *str2,
-					 gint	len2);
+                                         gint   len1,
+                                         gchar *str2,
+                                         gint   len2);
 typedef GValue (* TrackerDBFunc) (TrackerDBInterface *interface,
-				  gint		      argc,
-				  GValue	      argv[]);
-
-typedef void (* TrackerDBFuncStep) (TrackerDBInterface *interface,
-				    void               *aggregate_context,
-				    gint		  argc,
-				    GValue	          argv[]);
-
-typedef GValue (* TrackerDBFuncFinal) (TrackerDBInterface *interface,
-				       void               *aggregate_context);
+                                  gint                argc,
+                                  GValue              argv[]);
 
 struct TrackerDBInterfaceSqlite {
 	GObject parent_instance;
@@ -71,17 +62,13 @@ void                tracker_db_interface_sqlite_create_function        (TrackerD
                                                                         const gchar              *name,
                                                                         TrackerDBFunc             func,
                                                                         gint                      n_args);
-void                tracker_db_interface_sqlite_create_aggregate       (TrackerDBInterface       *interface,
-                                                                        const gchar              *name,
-                                                                        TrackerDBFuncStep         step,
-                                                                        gint                      n_args,
-                                                                        TrackerDBFuncFinal        final,
-                                                                        guint                     context_size);
 gboolean            tracker_db_interface_sqlite_set_collation_function (TrackerDBInterfaceSqlite *interface,
                                                                         const gchar              *name,
                                                                         TrackerDBCollationFunc    func);
 gint64              tracker_db_interface_sqlite_get_last_insert_id     (TrackerDBInterfaceSqlite *interface);
 void                tracker_db_interface_sqlite_enable_shared_cache    (void);
+void                tracker_db_interface_sqlite_fts_init               (TrackerDBInterfaceSqlite *interface,
+                                                                        gboolean                  create);
 
 G_END_DECLS
 
