@@ -1,27 +1,29 @@
 /*
  * Copyright (C) 2009, Nokia
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA  02110-1301, USA.
- *
- * Author: Philip Van Hoof <philip@codeminded.be>
  */
 
 #ifndef __LIBTRACKER_EXTRACT_EXIF_H__
 #define __LIBTRACKER_EXTRACT_EXIF_H__
 
+#if !defined (__LIBTRACKER_EXTRACT_INSIDE__) && !defined (TRACKER_COMPILATION)
+#error "only <libtracker-extract/tracker-extract.h> must be included directly."
+#endif
+
 #include <glib.h>
 
 G_BEGIN_DECLS
@@ -58,4 +60,3 @@ gboolean tracker_exif_read (const unsigned char *buffer,
 G_END_DECLS
 
 #endif /* __LIBTRACKER_EXTRACT_EXIF_H__ */
-
