@@ -1,18 +1,18 @@
 /*
  * Copyright (C) 2006, Mr Jamie McCracken (jamiemcc@gnome.org)
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA  02110-1301, USA.
  */
@@ -20,15 +20,11 @@
 #include "config.h"
 
 #include <locale.h>
-#include <string.h>
 
-#include <glib.h>
-
-#include <libtracker-common/tracker-type-utils.h>
 #include <libtracker-common/tracker-utils.h>
-#include <libtracker-common/tracker-ontology.h>
 
 #include "tracker-xmp.h"
+#include "tracker-utils.h"
 
 #ifdef HAVE_EXEMPI
 
@@ -37,9 +33,10 @@
 
 /**
  * SECTION:tracker-xmp
+ * @title: XMP
  * @short_description: Extensible Metadata Platform (XMP)
  * @stability: Stable
- * @include: libtracker-extract/tracker-xmp.h
+ * @include: libtracker-extract/tracker-extract.h
  *
  * The Adobe Extensible Metadata Platform (XMP) is a standard, created
  * by Adobe Systems Inc., for processing and storing standardized and
@@ -114,7 +111,7 @@ fix_flash (const gchar *flash)
 static const gchar *
 fix_white_balance (const gchar *wb)
 {
-	if (wb) {
+	if (g_strcmp0 (wb, "1") == 0) {
 		return "nmm:white-balance-manual";
 	} else {
 		return "nmm:white-balance-auto";
@@ -265,10 +262,10 @@ iterate_simple (const gchar    *uri,
 		if (!data->title2 && g_ascii_strcasecmp (name, "Title") == 0) {
 			data->title2 = g_strdup (value);
 		} else if (g_ascii_strcasecmp (name, "DateTimeOriginal") == 0 && !data->time_original) {
-			data->time_original = g_strdup (value);
+			data->time_original = tracker_date_guess (value);
 		} else if (!data->artist && g_ascii_strcasecmp (name, "Artist") == 0) {
 			data->artist = g_strdup (value);
-			/* } else if (g_ascii_strcasecmp (name, "Software") == 0) {
+		/* } else if (g_ascii_strcasecmp (name, "Software") == 0) {
 			   tracker_statement_list_insert (metadata, uri,
 			   "Image:Software", value);*/
 		} else if (!data->make && g_ascii_strcasecmp (name, "Make") == 0) {
@@ -281,7 +278,7 @@ iterate_simple (const gchar    *uri,
 			data->flash = g_strdup (fix_flash (value));
 		} else if (!data->metering_mode && g_ascii_strcasecmp (name, "MeteringMode") == 0) {
 			data->metering_mode = g_strdup (fix_metering_mode (value));
-			/* } else if (g_ascii_strcasecmp (name, "ExposureProgram") == 0) {
+		/* } else if (g_ascii_strcasecmp (name, "ExposureProgram") == 0) {
 			   tracker_statement_list_insert (metadata, uri,
 			   "Image:ExposureProgram", value);*/
 		} else if (!data->exposure_time && g_ascii_strcasecmp (name, "ExposureTime") == 0) {
@@ -316,7 +313,7 @@ iterate_simple (const gchar    *uri,
 		} else if (!data->description && g_ascii_strcasecmp (name, "description") == 0) {
 			data->description = g_strdup (value);
 		} else if (!data->date && g_ascii_strcasecmp (name, "date") == 0) {
-			data->date = g_strdup (value);
+			data->date = tracker_date_guess (value);
 		} else if (!data->keywords && g_ascii_strcasecmp (name, "keywords") == 0) {
 			data->keywords = g_strdup (value);
 		} else if (!data->subject && g_ascii_strcasecmp (name, "subject") == 0) {
@@ -350,8 +347,8 @@ iterate_simple (const gchar    *uri,
 		 * used sanely */
 
 		/* Photoshop TODO: is this needed anyway? */
-	} else if (g_ascii_strcasecmp (schema,  NS_PHOTOSHOP) == 0) {
-		if (data->city && g_ascii_strcasecmp (name, "City") == 0) {
+	} else if (g_ascii_strcasecmp (schema, NS_PHOTOSHOP) == 0) {
+		if (!data->city && g_ascii_strcasecmp (name, "City") == 0) {
 			data->city = g_strdup (value);
 		} else if (!data->country && g_ascii_strcasecmp (name, "Country") == 0) {
 			data->country = g_strdup (value);
@@ -361,7 +358,7 @@ iterate_simple (const gchar    *uri,
 			data->address = g_strdup (value);
 		}
 		/* IPTC4XMP scheme - GeoClue / location stuff, TODO */
-	} else if (g_ascii_strcasecmp (schema,  NS_IPTC4XMP) == 0) {
+	} else if (g_ascii_strcasecmp (schema, NS_IPTC4XMP) == 0) {
 		if (!data->city && g_ascii_strcasecmp (name, "City") == 0) {
 			data->city = g_strdup (value);
 		} else if (!data->country && g_ascii_strcasecmp (name, "Country") == 0) {
@@ -377,20 +374,12 @@ iterate_simple (const gchar    *uri,
 		} else if (!data->address && g_ascii_strcasecmp (name, "Sublocation") == 0) {
 			data->address = g_strdup (value);
 		}
-	} 
-	/* else
-	 * XAP (XMP)scheme *
-	 if (g_ascii_strcasecmp (schema, NS_XAP) == 0) {
-	 if (g_ascii_strcasecmp (name, "Rating") == 0) {
-	 tracker_statement_list_insert (metadata, uri,
-	 "Image:Rating", value);
-	 }
-	 if (g_ascii_strcasecmp (name, "MetadataDate") == 0) {
-	 tracker_statement_list_insert (metadata, uri,
-	 "Image:Date", value);
-	 }
-	 } else
-	*/
+	} else if (g_ascii_strcasecmp (schema, NS_XAP) == 0) {
+		if (!data->rating && g_ascii_strcasecmp (name, "Rating") == 0) {
+			data->rating = g_strdup (value);
+		}
+	}
+
 
 	g_free (name);
 }
